from django.conf import settings
from appconf import AppConf


class PagesExtAppConf(AppConf):
    FILE_LOCATION = getattr(settings, 'PAGES_EXT_FILE_LOCATION', settings.MEDIA_ROOT)
    FILE_UPLOAD_PERMISSIONS = getattr(
        settings, 'PAGES_EXT_FILE_UPLOAD_PERMISSIONS', settings.FILE_UPLOAD_PERMISSIONS)

    FILE_UPLOAD_DIRECTORY_PERMISSIONS = getattr(
            settings, 'PAGES_EXT_FILE_UPLOAD_DIRECTORY_PERMISSIONS', settings.FILE_UPLOAD_DIRECTORY_PERMISSIONS)

    FILE_OVERWRITE_EXISTS = getattr(settings, 'PAGES_EXT_FILE_OVERWRITE_EXISTS', True)

    IMAGE_DIR = getattr(settings, 'PAGES_EXT_IMAGE_DIR', 'pages/images')
    IMAGE_WIDTH_MAX = getattr(settings, 'PAGES_EXT_IMAGE_WIDTH_MAX', 600)
    IMAGE_HEIGHT_MAX = getattr(settings, 'PAGES_EXT_IMAGE_HEIGHT_MAX', 800)
    IMAGE_USE_ORIGINAL_FILENAME = getattr(settings, 'PAGE_EXT_IMAGE_USE_ORIGINAL_FILENAME', False)
    DELETE_IMAGE_FILE = getattr(settings, 'PAGES_EXT_DELETE_IMAGE_FILE', False)

    FILE_DIR = getattr(settings, 'PAGES_EXT_FILE_DIR', 'pages/files')
    FILE_USE_ORIGINAL_FILENAME = getattr(settings, 'PAGE_EXT_FILE_USE_ORIGINAL_FILENAME', False)
    FILE_DELETE_FILE = getattr(settings, 'PAGE_EXT_FILE_DELETE_FILE', False)

    class Meta:
        prefix = 'pages_ext'



"""
WSGI config for example_app project.

It exposes the WSGI callable as a module-level variable named ``application``.
"""

import os

from django.core.wsgi import get_wsgi_application

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "example_app.settings")

application = get_wsgi_application()



# -*- coding: utf-8
from __future__ import unicode_literals
from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse
from django.template import RequestContext
from django.utils import translation

from pages.cache import cache
from pages.models import PageSlugContent, PageMetaContent
from pages.models import PageTextContent
from pages.views import PageDetailsView
from pagesext.tests.base import PagesExtCase
from pagesext.models import PageTagsContent, PageImageContent, PageVideoContent, PageFileContent


class TestExtPages(PagesExtCase):
    def test_page_tags_model(self):
        PageTagsContent.objects.create(page=self.page_foo)
        obj = PageTagsContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:tags:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:tags:1')

    def test_page_tags_api(self):
        PageTagsContent.objects.create(page=self.page_foo)
        obj = PageTagsContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:tags:1')
        obj.add('tag1', 'tag2')
        names = sorted(list(obj.tags.names()))
        self.assertEqual(names[0], 'tag1')
        self.assertEqual(names[1], 'tag2')
        obj.set('tag3', 'tag4')
        names = sorted(list(obj.tags.names()))
        self.assertEqual(names[0], 'tag3')
        self.assertEqual(names[1], 'tag4')
        slugs = sorted(list(obj.tags.slugs()))
        self.assertEqual(slugs[0], 'tag3')
        self.assertEqual(slugs[1], 'tag4')
        PageTagsContent.objects.create(page=self.page_foo2)
        obj2 = PageTagsContent.objects.filter(page=self.page_foo2, language='en')[0]
        obj2.add('tag4', 'tag5')
        objects = obj.similar_objects()
        self.assertEqual(objects[0], obj2)
        obj.remove('tag3')
        names = obj.tags.names()
        self.assertEqual(len(names), 1)
        obj.clear()
        names = obj.tags.names()
        self.assertEqual(len(names), 0)

    def test_page_image_model(self):
        PageImageContent.objects.create(page=self.page_foo)
        obj = PageImageContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:image:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:image:1')

    def test_page_video_model(self):
        PageVideoContent.objects.create(page=self.page_foo)
        obj = PageVideoContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:video:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:video:1')

    def test_page_file_model(self):
        PageFileContent.objects.create(page=self.page_foo)
        obj = PageFileContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:file:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:file:1')

    def test_page_tags_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        PageTagsContent.objects.create(page=self.page_foo)
        obj = PageTagsContent.objects.filter(page=self.page_foo, language='en')[0]
        obj.add('tag1', 'tag2')
        self.page_foo.template = 'pages/page_tags.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()

    def test_page_image_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        PageImageContent.objects.create(page=self.page_foo, image='img/test.jpg', title='test')

        img1 = PageImageContent.objects.create(page=self.page_foo, image='img/test.jpg', title='test')
        img1.tags.add('test', 'image-1')
        img2 = PageImageContent.objects.create(page=self.page_foo, image='img/test.jpg', title='test')
        img2.tags.add('test', 'image-2')

        self.page_foo.template = 'pages/page_image.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')

        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()

    def test_page_video_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        PageVideoContent.objects.create(
            page=self.page_foo, video='https://www.youtube.com/watch?v=C0DPdy98e4c', title='test'
        )

        video1 = PageVideoContent.objects.create(
            page=self.page_foo, video='https://www.youtube.com/watch?v=C0DPdy98e4c', title='test'
        )
        video1.tags.add('test', 'video-1')
        video2 = PageVideoContent.objects.create(
            page=self.page_foo, video='https://www.youtube.com/watch?v=C0DPdy98e4c', title='test'
        )
        video2.tags.add('test', 'video-2')

        self.page_foo.template = 'pages/page_video.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()

    def test_page_file_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        PageFileContent.objects.create(page=self.page_foo, file='files/test.txt', title='test')

        file1 = PageFileContent.objects.create(page=self.page_foo, file='files/test.txt', title='test')
        file1.tags.add('test', 'file-1')
        file2 = PageFileContent.objects.create(page=self.page_foo, file='files/test.txt', title='test')
        file2.tags.add('test', 'file-2')

        self.page_foo.template = 'pages/page_file.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()



from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.test import TestCase
from django.test.client import RequestFactory

from pages.models.page import Page


class PagesExtCase(TestCase):
    urls = 'pagesext.tests.urls'
    fixtures = ['fixtures/pagesext_testdata.yaml']

    def setUp(self):
        self.factory = RequestFactory()
        self.site_foo = Site.objects.get(id=1)
        self.user_foo = User.objects.create(username='foo', password='bar', email='foo@test.com')
        self.page_foo = Page.objects.create(name='Test', created_by=self.user_foo,
                                            updated_by=self.user_foo, is_draft=False, is_published=True)

        self.page_foo2 = Page.objects.create(name='Test2', created_by=self.user_foo,
                                             updated_by=self.user_foo, is_draft=False, is_published=True)

    def tearDown(self):
        self.user_foo.delete()



"""Implements page tags extended content model"""

from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from taggit.managers import TaggableManager

from pages.conf import settings
from pages.models.pagebasecontent import PageBaseContent

if settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:

    class PageTagsContent(PageBaseContent):
        tags = TaggableManager()
        objects = models.Manager()

        def add(self, *args):
            return self.tags.add(*args)

        def remove(self, *args):
            return self.tags.remove(*args)

        def set(self, *args):
            return self.tags.set(*args)

        def clear(self):
            return self.tags.clear()

        def names(self):
            return self.tags.names()

        def similar_objects(self):
            return self.tags.similar_objects()

        def slugs(self):
            return self.tags.slugs()

        def update_fields(self, change):
            if not change:
                self.type = 'tags'
                self.is_extended = True

        class Meta(PageBaseContent.Meta):
            app_label = 'pagesext'
            verbose_name = _('Tags')
            verbose_name_plural = _('Tags')

        class PageMeta(PageBaseContent.PageMeta):
            context_name = 'tags'
            multiple_per_page = True
            multiple_per_locale = True



from pagesext.admin.pagetagscontent import PageTagsContentAdmin
from pagesext.admin.pageimagecontent import PageImageContentAdmin
from pagesext.admin.pagefilecontent import PageFileContentAdmin
from pagesext.admin.pagevideocontent import PageVideoContentAdmin



from django.contrib import admin

from image_cropping import ImageCroppingMixin

from pagesext.models import PageImageContent


class PageImageContentAdmin(ImageCroppingMixin, admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by',)
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageImageContent, PageImageContentAdmin)


class PageImageContentInline(admin.StackedInline):
    model = PageImageContent
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    readonly_fields = ['image_cropping_link']
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('image', ),
            ('image_cropping_link', ),
            ('title', ),
            ('tags', ),
            ('comment', ),
        ]}),
    ]



from pagesext.models.pagetagscontent import PageTagsContent
from pagesext.models.pageimagecontent import PageImageContent
from pagesext.models.pagevideocontent import PageVideoContent
from pagesext.models.pagefilecontent import PageFileContent



from django import template

from pagesext.models import PageImageContent, PageVideoContent, PageFileContent
from pages.templatetags.pages_tags import get_page_object_by_id

register = template.Library()


@register.simple_tag(takes_context=True)
def page_tags_by_id(context, oid):
    obj = get_page_object_by_id(context, 'tags', oid)
    return obj if obj else None


@register.assignment_tag(takes_context=True)
def get_page_tags_by_id(context, oid):
    return page_tags_by_id(context, oid)


@register.simple_tag(takes_context=True)
def page_image_by_id(context, oid):
    obj = get_page_object_by_id(context, 'image', oid)
    return obj if obj else None


@register.assignment_tag(takes_context=True)
def get_page_image_by_id(context, oid):
    return page_image_by_id(context, oid)


@register.simple_tag(takes_context=True)
def page_image_by_tag(context, tag):
    page = context['page']['page']
    images = PageImageContent.objects.filter(page=page, tags__name__in=[tag])
    return images[0] if len(images) else None


@register.assignment_tag(takes_context=True)
def get_page_image_by_tag(context, tag):
    page = context['page']['page']
    images = PageImageContent.objects.filter(page=page, tags__name__in=[tag])
    return images[0] if len(images) else None


@register.assignment_tag(takes_context=True)
def get_page_images_by_tags(context, tags):
    page = context['page']['page']
    images = PageImageContent.objects.filter(page=page, tags__name__in=[tags])
    return images


@register.simple_tag(takes_context=True)
def page_video_by_id(context, oid):
    obj = get_page_object_by_id(context, 'video', oid)
    return obj if obj else None


@register.assignment_tag(takes_context=True)
def get_page_video_by_id(context, oid):
    return page_video_by_id(context, oid)


@register.simple_tag(takes_context=True)
def page_video_by_tag(context, tag):
    page = context['page']['page']
    videos = PageVideoContent.objects.filter(page=page, tags__name__in=[tag])
    return videos[0] if len(videos) else None


@register.assignment_tag(takes_context=True)
def get_page_video_by_tag(context, tag):
    page = context['page']['page']
    videos = PageVideoContent.objects.filter(page=page, tags__name__in=[tag])
    return videos[0] if len(videos) else None


@register.assignment_tag(takes_context=True)
def get_page_videos_by_tags(context, tags):
    page = context['page']['page']
    videos = PageVideoContent.objects.filter(page=page, tags__name__in=[tags])
    return videos


@register.simple_tag(takes_context=True)
def page_file_by_id(context, oid):
    obj = get_page_object_by_id(context, 'file', oid)
    return obj if obj else None


@register.assignment_tag(takes_context=True)
def get_page_file_by_id(context, oid):
    return page_file_by_id(context, oid)


@register.simple_tag(takes_context=True)
def page_file_by_tag(context, tag):
    page = context['page']['page']
    files = PageFileContent.objects.filter(page=page, tags__name__in=[tag])
    return files[0] if len(files) else None


@register.assignment_tag(takes_context=True)
def get_page_file_by_tag(context, tag):
    page = context['page']['page']
    files = PageFileContent.objects.filter(page=page, tags__name__in=[tag])
    return files[0] if len(files) else None


@register.assignment_tag(takes_context=True)
def get_page_files_by_tags(context, tags):
    page = context['page']['page']
    files = PageFileContent.objects.filter(page=page, tags__name__in=[tags])
    return files



"""Implements admin interface for page file content"""

from django.contrib import admin
from pages.conf import settings

if settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:
    from pagesext.models.pagefilecontent import PageFileContent

    class PageFileContentAdmin(admin.ModelAdmin):
        list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
        list_display_links = ['__str__']
        fieldsets = [
            (None, {'fields': [
                ('language',),
                ('name',),
                ('file',),
                ('title',),
                ('description',),
                ('tags',),
                ('comment',),
            ]}),
        ]

        exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by',)
        save_on_top = True
        actions_on_bottom = True

    admin.site.register(PageFileContent, PageFileContentAdmin)

    class PageFileContentInline(admin.StackedInline):
        model = PageFileContent
        extra = 1
        exclude = ('sid', 'is_extended', 'created_by', 'updated_by', 'date_created', 'date_updated',)
        fieldsets = [
            (None, {'fields': [
                ('language',),
                ('name',),
                ('file',),
                ('title',),
                ('description',),
                ('comment',),
            ]}),
        ]
